@@ -1,6 +1,7 @@
 """
 Chromecast constants
 """
+
 # Regular chromecast, supports video/audio
 CAST_TYPE_CHROMECAST = "cast"
 # Cast Audio device, supports only audio
@@ -8,6 +9,9 @@ CAST_TYPE_AUDIO = "audio"
 # Cast Audio group device, supports only audio
 CAST_TYPE_GROUP = "group"
 
+# Default command timeout
+REQUEST_TIMEOUT = 10.0
+
 MF_CANTON = "Canton Elektronik GmbH + Co. KG"
 MF_GOOGLE = "Google Inc."
 MF_JBL = "JBL"
@@ -59,8 +63,6 @@ CAST_TYPES = {
     "V705-H3": (CAST_TYPE_CHROMECAST, MF_VIZIO),
 }
 
-SERVICE_TYPE_HOST = "host"
-SERVICE_TYPE_MDNS = "mdns"
 
 MESSAGE_TYPE = "type"
 REQUEST_ID = "requestId"
